@@ -1,6 +1,6 @@
 # -*- coding: utf-8; mode: tcl; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- vim:fenc=utf-8:ft=tcl:et:sw=4:ts=4:sts=4
 # portuninstall.tcl
-# $Id: portuninstall.tcl 100223 2012-12-04 00:50:31Z cal@macports.org $
+# $Id: portuninstall.tcl 117407 2014-02-25 18:28:14Z jmr@macports.org $
 #
 # Copyright (c) 2004-2005, 2008-2011 The MacPorts Project
 # Copyright (c) 2002 - 2003 Apple Inc.
@@ -43,7 +43,7 @@ namespace eval registry_uninstall {
 
 # takes a composite version spec rather than separate version,revision,variants
 proc uninstall_composite {portname {v ""} {optionslist ""}} {
-    if {$v == ""} {
+    if {$v eq ""} {
         return [uninstall $portname "" "" 0 $optionslist]
     } elseif {[registry::decode_spec $v version revision variants]} {
         return [uninstall $portname $version $revision $variants $optionslist]
@@ -53,7 +53,7 @@ proc uninstall_composite {portname {v ""} {optionslist ""}} {
 
 proc uninstall {portname {version ""} {revision ""} {variants 0} {optionslist ""}} {
     global uninstall.force uninstall.nochecksum UI_PREFIX \
-           macports::portimagefilepath
+           macports::portimagefilepath macports::registry.path
     array set options $optionslist
     if {[info exists options(subport)]} {
         # don't want this set when calling registry::run_target
@@ -75,12 +75,12 @@ proc uninstall {portname {version ""} {revision ""} {variants 0} {optionslist ""
 
     set searchkeys $portname
     set composite_spec ""
-    if {$version != ""} {
+    if {$version ne ""} {
         lappend searchkeys $version
         set composite_spec $version
         # restriction imposed by underlying registry API (see entry.c):
         # if a revision is specified, so must variants be
-        if {$revision != ""} {
+        if {$revision ne ""} {
             lappend searchkeys $revision $variants
             append composite_spec _${revision}${variants}
         }
@@ -92,7 +92,7 @@ proc uninstall {portname {version ""} {revision ""} {variants 0} {optionslist ""
         ui_msg "$UI_PREFIX [msgcat::mc "The following versions of $portname are currently installed:"]"
         foreach i [portlist_sortint $ilist] {
             set ispec "[$i version]_[$i revision][$i variants]"
-            if { [string equal [$i state] installed] } {
+            if {[$i state] eq "installed"} {
                 ui_msg "$UI_PREFIX [format [msgcat::mc "    %s @%s (active)"] [$i name] $ispec]"
             } else {
                 ui_msg "$UI_PREFIX [format [msgcat::mc "    %s @%s"] [$i name] $ispec]"
@@ -106,7 +106,7 @@ proc uninstall {portname {version ""} {revision ""} {variants 0} {optionslist ""
         set variants [$port variants]
         set composite_spec "${version}_${revision}${variants}"
     } else {
-        if {$composite_spec != ""} {
+        if {$composite_spec ne ""} {
             set composite_spec " @${composite_spec}"
         }
         throw registry::invalid "Registry error: ${portname}${composite_spec} not registered as installed"
@@ -122,7 +122,7 @@ proc uninstall {portname {version ""} {revision ""} {variants 0} {optionslist ""
         }
         foreach depport [$port dependents] {
             # make sure it's still installed, since a previous dep uninstall may have removed it
-            if {[registry::entry exists $depport] && ([$depport state] == "imaged" || [$depport state] == "installed")} {
+            if {[registry::entry exists $depport] && ([$depport state] eq "imaged" || [$depport state] eq "installed")} {
                 if {[info exists options(ports_uninstall_no-exec)] || ![registry::run_target $depport uninstall $optionslist]} {
                     registry_uninstall::uninstall [$depport name] [$depport version] [$depport revision] [$depport variants] $optionslist
                 }
@@ -137,7 +137,7 @@ proc uninstall {portname {version ""} {revision ""} {variants 0} {optionslist ""
         registry::check_dependents $port ${uninstall.force} "uninstall"
     }
     # if it's active, deactivate it
-    if { [string equal [$port state] installed] } {
+    if {[$port state] eq "installed"} {
         if {[info exists options(ports_dryrun)] && [string is true -strict $options(ports_dryrun)]} {
             ui_msg "For $portname @${composite_spec}: skipping deactivate (dry run)"
         } else {
@@ -156,7 +156,7 @@ proc uninstall {portname {version ""} {revision ""} {variants 0} {optionslist ""
         # look up deps from the saved portfile if possible
         if {![catch {set mport [mportopen_installed [$port name] [$port version] [$port revision] [$port variants] $optionslist]}]} {
             array set depportinfo [mportinfo $mport]
-            mportclose_installed $mport
+            mportclose $mport
             foreach type $deptypes {
                 if {[info exists depportinfo($type)]} {
                     foreach dep $depportinfo($type) {
@@ -217,9 +217,37 @@ proc uninstall {portname {version ""} {revision ""} {variants 0} {optionslist ""
         # files so just ignore the failure
         catch {file delete [::file dirname $imagefile]}
 
+        # We want to delete the portfile if not referenced by any other ports
+        set portfile [$ref portfile]
+
+        # and likewise the portgroups
+        set portgroups [list]
+        foreach pg [$ref groups_used] {
+            lappend portgroups [list [$pg name] [$pg version] [$pg size] [$pg sha256]]
+        }
+
         registry::write {
             registry::entry delete $port
         }
+
+        set portfile_path [file join ${registry.path} registry portfiles ${portname}-${version}_${revision} $portfile]
+        if {[registry::entry search portfile $portfile] eq {}} {
+            file delete -force $portfile_path
+            catch {file delete [file dirname $portfile_path]}
+        }
+
+        set reg_portgroups_dir [file join ${registry.path} registry portgroups]
+        foreach pg $portgroups {
+            set pgname [lindex $pg 0]
+            set pgversion [lindex $pg 1]
+            set pgsize [lindex $pg 2]
+            set pgsha256 [lindex $pg 3]
+            if {[registry::portgroup search name $pgname version $pgversion size $pgsize sha256 $pgsha256] eq {}} {
+                set pg_reg_dir [file join $reg_portgroups_dir ${pgsha256}-${pgsize}]
+                file delete -force ${pg_reg_dir}/${pgname}-${pgversion}.tcl
+                catch {file delete $pg_reg_dir}
+            }
+        }
     }
     
     # uninstall dependencies if requested
